@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2009-2010 Nick Schermer <nick@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -192,6 +192,9 @@ panel_base_window_class_init (PanelBaseWindowClass *klass)
 static void
 panel_base_window_init (PanelBaseWindow *window)
 {
+  GdkColormap *colormap;
+  GdkScreen   *screen;
+
   window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, PANEL_TYPE_BASE_WINDOW, PanelBaseWindowPrivate);
 
   window->is_composited = FALSE;
@@ -206,7 +209,23 @@ panel_base_window_init (PanelBaseWindow *window)
   window->priv->borders = PANEL_BORDER_NONE;
   window->priv->active_timeout_id = 0;
 
-  panel_base_window_composited_changed (GTK_WIDGET (window));
+  /* some wm require stick to show the window on all workspaces, on xfwm4
+   * the type-hint already takes care of that */
+  gtk_window_stick (GTK_WINDOW (window));
+
+  /* set the rgba colormap if supported by the screen */
+  screen = gtk_window_get_screen (GTK_WINDOW (window));
+  colormap = gdk_screen_get_rgba_colormap (screen);
+  if (colormap != NULL)
+    {
+      gtk_widget_set_colormap (GTK_WIDGET (window), colormap);
+      window->is_composited = gtk_widget_is_composited (GTK_WIDGET (window));
+    }
+
+   panel_debug (PANEL_DEBUG_BASE_WINDOW,
+               "%p: rgba colormap=%s, compositing=%s", window,
+               PANEL_DEBUG_BOOL (colormap != NULL),
+               PANEL_DEBUG_BOOL (window->is_composited));
 }
 
 
@@ -442,19 +461,16 @@ panel_base_window_expose_event (GtkWidget      *widget,
   gdouble                 height = widget->allocation.height;
   const gdouble           dashes[] = { 4.00, 4.00 };
   GTimeVal                timeval;
-  gboolean                result;
   GdkPixbuf              *pixbuf;
   GError                 *error = NULL;
   cairo_matrix_t          matrix = { 1, 0, 0, 1, 0, 0 }; /* identity matrix */
 
-  result = (*GTK_WIDGET_CLASS (panel_base_window_parent_class)->expose_event) (widget, event);
-
   if (!GTK_WIDGET_DRAWABLE (widget))
-    return result;
+    return FALSE;
 
   /* create cairo context and set some default properties */
   cr = gdk_cairo_create (widget->window);
-  panel_return_val_if_fail (cr != NULL, result);
+  panel_return_val_if_fail (cr != NULL, FALSE);
   cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
   cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
   cairo_set_line_width (cr, 1.00);
@@ -595,7 +611,7 @@ panel_base_window_expose_event (GtkWidget      *widget,
 
   cairo_destroy (cr);
 
-  return result;
+  return FALSE;
 }
 
 
@@ -640,50 +656,17 @@ static void
 panel_base_window_composited_changed (GtkWidget *widget)
 {
   PanelBaseWindow *window = PANEL_BASE_WINDOW (widget);
-  GdkColormap     *colormap = NULL;
   gboolean         was_composited = window->is_composited;
-  gboolean         was_visible;
-  GdkScreen       *screen;
-  gboolean         colormap_changed;
-  gboolean         colormap_rgba = FALSE;
+  GdkWindow       *gdkwindow;
 
-  panel_return_if_fail (PANEL_IS_BASE_WINDOW (widget));
+  /* set new compositing state */
+  window->is_composited = gtk_widget_is_composited (widget);
+  if (window->is_composited)
+    gtk_window_set_opacity (GTK_WINDOW (widget), window->priv->leave_opacity);
 
-  /* get the widget screen */
-  screen = gtk_window_get_screen (GTK_WINDOW (widget));
-  panel_return_if_fail (GDK_IS_SCREEN (screen));
-
-  /* get the rgba colormap */
-  colormap = gdk_screen_get_rgba_colormap (screen);
-  if (G_UNLIKELY (colormap == NULL))
-    {
-      window->is_composited = FALSE;
-      colormap = gdk_screen_get_rgb_colormap (screen);
-    }
-  else
-    {
-      colormap_rgba = TRUE;
-      window->is_composited = gtk_widget_is_composited (widget);
-      gtk_window_set_opacity (GTK_WINDOW (widget), window->priv->leave_opacity);
-    }
-
-  panel_return_if_fail (GDK_IS_COLORMAP (colormap));
-  colormap_changed = gtk_widget_get_colormap (widget) != colormap;
-
-  panel_debug (PANEL_DEBUG_DOMAIN_BASE_WINDOW,
-               "set new colormap; composited=%s, rgba=%s, visible=%s",
-               PANEL_DEBUG_BOOL (gtk_widget_is_composited (widget)),
-               PANEL_DEBUG_BOOL (colormap_rgba),
-               PANEL_DEBUG_BOOL (GTK_WIDGET_VISIBLE (widget)));
-
-  was_visible = GTK_WIDGET_VISIBLE (widget);
-  if (was_visible)
-    {
-      gtk_widget_hide (widget);
-
-      if (colormap_changed)
-        gtk_widget_unrealize (widget);
-    }
+  panel_debug (PANEL_DEBUG_BASE_WINDOW,
+               "%p: compositing=%s", window,
+               PANEL_DEBUG_BOOL (window->is_composited));
 
   /* clear cairo image cache */
   if (window->priv->bg_image_cache != NULL)
@@ -692,20 +675,19 @@ panel_base_window_composited_changed (GtkWidget *widget)
       window->priv->bg_image_cache = NULL;
     }
 
-  if (colormap_changed)
-    gtk_widget_set_colormap (widget, colormap);
-
-  if (was_visible)
-    {
-      /* restore the window */
-      if (colormap_changed)
-        gtk_widget_realize (widget);
-      gtk_widget_show (widget);
-    }
-
-  /* emit the property if it changed */
   if (window->is_composited != was_composited)
     g_object_notify (G_OBJECT (widget), "composited");
+
+  /* make sure the entire window is redrawn */
+  gdkwindow = gtk_widget_get_window (widget);
+  if (gdkwindow != NULL)
+    gdk_window_invalidate_rect (gdkwindow, NULL, TRUE);
+
+  /* HACK: invalid the geometry, so the wm notices it */
+  gtk_window_move (GTK_WINDOW (window),
+                   widget->allocation.x,
+                   widget->allocation.y);
+  gtk_widget_queue_resize (widget);
 }
 
 
