@@ -35,7 +35,10 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4panel/libxfce4panel.h>
+
 #include <common/panel-private.h>
+#include <common/panel-debug.h>
+
 #include "systray-socket.h"
 
 
@@ -119,39 +122,44 @@ static void
 systray_socket_realize (GtkWidget *widget)
 {
   SystraySocket *socket = XFCE_SYSTRAY_SOCKET (widget);
-  GdkVisual     *visual;
   GdkColor       transparent = { 0, 0, 0, 0 };
+  GdkWindow     *window;
 
   GTK_WIDGET_CLASS (systray_socket_parent_class)->realize (widget);
 
-  visual = gtk_widget_get_visual (widget);
-  if (visual->red_prec + visual->blue_prec + visual->green_prec < visual->depth
-      && gdk_display_supports_composite (gtk_widget_get_display (widget)))
+  window = gtk_widget_get_window (widget);
+
+  if (socket->is_composited)
     {
-      gdk_window_set_background (widget->window, &transparent);
-      gdk_window_set_composited (widget->window, TRUE);
+      gdk_window_set_background (window, &transparent);
+      gdk_window_set_composited (window, TRUE);
 
-      socket->is_composited = TRUE;
       socket->parent_relative_bg = FALSE;
     }
-  else if (visual == gdk_drawable_get_visual (
-               GDK_DRAWABLE (gdk_window_get_parent (widget->window))))
+  else if (gtk_widget_get_visual (widget) ==
+           gdk_drawable_get_visual (GDK_DRAWABLE (gdk_window_get_parent (window))))
     {
-      gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
+      gdk_window_set_back_pixmap (window, NULL, TRUE);
 
-      socket->is_composited = FALSE;
       socket->parent_relative_bg = TRUE;
     }
   else
     {
-      socket->is_composited = FALSE;
       socket->parent_relative_bg = FALSE;
     }
 
+  gdk_window_set_composited (window, socket->is_composited);
+
   gtk_widget_set_app_paintable (widget,
       socket->parent_relative_bg || socket->is_composited);
 
   gtk_widget_set_double_buffered (widget, socket->parent_relative_bg);
+
+  panel_debug_filtered (PANEL_DEBUG_SYSTRAY,
+      "socket %s[%p] (composited=%s, relative-bg=%s",
+      systray_socket_get_name (socket), socket,
+      PANEL_DEBUG_BOOL (socket->is_composited),
+      PANEL_DEBUG_BOOL (socket->parent_relative_bg));
 }
 
 
@@ -166,7 +174,8 @@ systray_socket_size_allocate (GtkWidget     *widget,
   gboolean       resized = allocation->width != widget->allocation.width
                            ||allocation->height != widget->allocation.height;
 
-  if ((moved || resized) && GTK_WIDGET_MAPPED (widget))
+  if ((moved || resized)
+      && GTK_WIDGET_MAPPED (widget))
     {
       if (socket->is_composited)
         gdk_window_invalidate_rect (gdk_window_get_parent (widget->window),
@@ -175,7 +184,8 @@ systray_socket_size_allocate (GtkWidget     *widget,
 
   GTK_WIDGET_CLASS (systray_socket_parent_class)->size_allocate (widget, allocation);
 
-  if ((moved || resized) && GTK_WIDGET_MAPPED (widget))
+  if ((moved || resized)
+      && GTK_WIDGET_MAPPED (widget))
     {
       if (socket->is_composited)
         gdk_window_invalidate_rect (gdk_window_get_parent (widget->window),
@@ -186,6 +196,7 @@ systray_socket_size_allocate (GtkWidget     *widget,
 }
 
 
+
 static gboolean
 systray_socket_expose_event (GtkWidget      *widget,
                              GdkEventExpose *event)
@@ -208,9 +219,9 @@ systray_socket_expose_event (GtkWidget      *widget,
       /* clear to parent-relative pixmap */
       gdk_window_clear_area (widget->window,
                              event->area.x,
-           event->area.y,
+                             event->area.y,
                              event->area.width,
-           event->area.height);
+                             event->area.height);
     }
 
   return FALSE;
@@ -246,12 +257,13 @@ systray_socket_new (GdkScreen       *screen,
   result = XGetWindowAttributes (GDK_DISPLAY_XDISPLAY (display),
                                  window, &attr);
 
-  /* leave on an error or is the window does not exist */
+  /* leave on an error or if the window does not exist */
   if (gdk_error_trap_pop () != 0 || result == 0)
     return NULL;
 
   /* get the windows visual */
   visual = gdk_x11_screen_lookup_visual (screen, attr.visual->visualid);
+  panel_return_val_if_fail (visual == NULL || GDK_IS_VISUAL (visual), NULL);
   if (G_UNLIKELY (visual == NULL))
     return NULL;
 
@@ -271,13 +283,19 @@ systray_socket_new (GdkScreen       *screen,
 
   /* create a new socket */
   socket = g_object_new (XFCE_TYPE_SYSTRAY_SOCKET, NULL);
-  gtk_widget_set_colormap (GTK_WIDGET (socket), colormap);
   socket->window = window;
+  socket->is_composited = FALSE;
+  gtk_widget_set_colormap (GTK_WIDGET (socket), colormap);
 
   /* release the custom colormap */
   if (release_colormap)
     g_object_unref (G_OBJECT (colormap));
 
+  /* check if there is an alpha channel in the visual */
+  if (visual->red_prec + visual->blue_prec + visual->green_prec < visual->depth
+      && gdk_display_supports_composite (gdk_screen_get_display (screen)))
+    socket->is_composited = TRUE;
+
   return GTK_WIDGET (socket);
 }
 
@@ -385,7 +403,7 @@ systray_socket_get_name_prop (SystraySocket *socket,
 
 
 const gchar *
-systray_socket_get_name  (SystraySocket *socket)
+systray_socket_get_name (SystraySocket *socket)
 {
   panel_return_val_if_fail (XFCE_IS_SYSTRAY_SOCKET (socket), NULL);
 
